unit IICalcIntf;

{*******************************************************}
{                                                       }
{       IICalc SDK                                      }
{       Interfaces                                      }
{                                                       }
{       2002 Demoonlit                                  }
{                                                       }
{*******************************************************}

interface

uses
	Windows,
	{LInterfaces,}
	IICalcShareMem;

type
	TNumberShowing = (ns2, ns10, ns16);
	TOperatorMethod = (omPrefix, omInfix, omPostfix);

	TDescription = record
		NumberShowing: TNumberShowing;
		OperatorMethod: TOperatorMethod;
	end;

	PType = ^TType;
	TType = record
		Name: WideString;
		Supers: Pointer;
		Kind: (tkAny, tkArray, tkSet);
		Element: PType;
	end;

type
	IValue = interface(IInterface)
		['{EFED2D12-2F1B-43D1-8570-94959A4CF051}']
		function GetType: PType;
		function ToString(const Description: TDescription): WideString;
	end;

	IIntegerValue = interface(IValue)
		['{39034145-A905-474E-BE47-109C21A3F941}']
		function GetValue: Integer;
		property Value: Integer read GetValue;
	end;

	IRealValue = interface(IValue)
		['{473EDF59-1E1D-4B90-A177-A89E3EBF2C66}']
		function GetValue: Extended;
		property Value: Extended read GetValue;
	end;

	IComplexValue = interface(IValue)
		['{85B2713B-9A50-4024-ACE9-60F7E3998620}']
		function GetRealValue: Extended;
		property RealValue: Extended read GetRealValue;
		function GetImaginaryValue: Extended;
		property ImaginaryValue: Extended read GetImaginaryValue;
	end;

	ILetterValue = interface(IValue)
		['{E6F20FFD-B559-4FAD-BF01-4BFF5912706E}']
		function GetValue: WideChar;
		property Value: WideChar read GetValue;
	end;

	IBooleanValue = interface(IValue)
		['{3E7B0229-6929-4978-8F20-F6734AC01ED0}']
		function GetValue: Boolean;
		property Value: Boolean read GetValue;
	end;

	IOrdinalValue = interface(IValue)
		['{E7A6F444-CF41-4208-8A96-7CF68C391FB4}']
		function GetValue: Integer;
		property Value: Integer read GetValue;
	end;

	IArrayValue = interface(IValue)
		['{B2B77004-96B0-42FC-A8A5-CB808A701806}']
		function GetAt(AIndex: Integer): IValue;
		property At[AIndex: Integer]: IValue read GetAt;
		function GetCount: Integer;
		property Count: Integer read GetCount;
	end;

type
	TOperatorPriority = (prMonad, prFlow, prLogic, prSum, prProduct, prPower, prFunc, prClause);

	IEnvironmentFrame = interface(IInterface)
		['{063B001F-62C3-4849-828C-478D54C4C289}']
		function AddBinding(const AName: WideString; AType: PType): Integer;
		function FindBinding(const AName: WideString): Integer;
		procedure HideAfter(FromIndex: Integer);
		procedure Hide(Index: Integer);
		procedure Slice(FromIndex: Integer);
		function GetCount: Integer;
		property Count: Integer read GetCount;
		function GetNames(AIndex: Integer): WideString;
		property Names[AIndex: Integer]: WideString read GetNames;
		function GetTypes(AIndex: Integer): PType;
		property Types[AIndex: Integer]: PType read GetTypes;
	end;

	IEnvironment = interface(IInterface)
		['{5091E3EC-E7FF-44AD-AB26-15A3847BDD65}']
		function GetValues(AIndex: Integer): IValue;
		procedure SetValues(AIndex: Integer; const Value: IValue);
		property Values[AIndex: Integer]: IValue read GetValues write SetValues;
	end;

	IExpression = interface(IInterface)
		['{289D4DA1-4538-4082-B690-82075FED4BDD}']
		function GetType: PType;
		function Execute(const Environment: IEnvironment): IValue;
		function ToString(Method: TOperatorMethod): WideString;
		function GetPriority: TOperatorPriority;
		property Priority: TOperatorPriority read GetPriority;
		{Utilities for Compile, DO NOT USE}
		procedure HideBindings;
	end;

	TFunctionProc = function(const Arguments: array of IExpression; const Environment: IEnvironment): IValue of object;

	PFunction = ^TFunction;
	TFunction = record
		Name: WideString;
		Execute: TFunctionProc;
		ArgumentTypes: Pointer;
		ResultType: PType;
		IsMethod: Boolean;
	end;

	ITemplateManager = interface(IInterface)
		['{6DAF7FF2-1A7E-4528-A08B-7A111CC4C2ED}']
		procedure RegisterType(AType: PType);
		procedure RegisterFunction(AFunc: PFunction);
	end;

	IValueManager = interface(IInterface)
		['{79B31568-3135-4D80-8EFF-664521E8A149}']
		function RealType: PType;
		function CreateRealValue(const Value: Extended): IRealValue;
	end;

function GetApplication: IInterface; external Core name 'IICalc@GetApplication';
function GetCurrentThread: IInterface; external Core name 'IICalc@GetCurrentThread';

implementation

end.
